InteractionFlags

Struct InteractionFlags 

pub struct InteractionFlags {
    pub focusable: bool,
    pub scrollable: bool,
    pub draggable: bool,
}
Flags describing how an interactive region participates in input/focus.

Fields

focusable: bool

Region can receive focus (e.g. clicks set focus).

scrollable: bool

Region should react to mouse wheel scrolling if hovered.

draggable: bool

Region can be dragged (used by sliders/scrollbars, etc.).

Implementations

Source

impl InteractionFlags

pub const fn new(focusable: bool, scrollable: bool, draggable: bool) -> Self

pub const fn none() -> Self

pub const fn focusable() -> Self

pub const fn scrollable() -> Self

pub const fn draggable() -> Self

pub const fn union(self, other: Self) -> Self