pub struct InteractionFlags {
pub focusable: bool,
pub scrollable: bool,
pub draggable: bool,
}Expand description
Flags describing how an interactive region participates in input/focus.
Fields
focusable: boolRegion can receive focus (e.g. clicks set focus).
scrollable: boolRegion should react to mouse wheel scrolling if hovered.
draggable: boolRegion can be dragged (used by sliders/scrollbars, etc.).
Implementations